local addonName, addonTable = ...
local AceConfig = _G.LibStub("AceConfig-3.0")
local AceConfigDialog = _G.LibStub("AceConfigDialog-3.0")
local E = addonTable.E
local L = addonTable.L
local wipe = _G.wipe
local pairs = _G.pairs
local GetTalentInfoBySpecialization = _G.GetTalentInfoBySpecialization
local CreateFrame = _G.CreateFrame

E.config = {}

function E.config:getTalentsForSpec(spec, tier)
    local t = {}
    for i=1, 3 do
        local id, name = GetTalentInfoBySpecialization(spec, tier, i)
        t[i] = name
    end
    return t
end

function E.config:getSpecTable()
    local specTable = {}
    specTable[0] = L["Don't switch"]
    for i= 1, #E.specTable do
        specTable[i] = E.specTable[i].name
    end
    return specTable
end

function E.config:refreshOptions()
    E:gatherEquipmentSets()
    -- todo: parse options table (memory leak)
    self:buildOptionsTable()
end

function E.config:buildOptionsTable()
    if self.optionsTable then
        wipe(self.optionsTable.args)
    else
        self.optionsTable = {}
        self.optionsTable.type = "group"
        self.optionsTable.args = {}
    end

    local args = self.optionsTable.args
    for k, v in pairs(E.equipmentSets) do
        args[k] = {
            name = k,
            type = "group",
            args = {
                select = {
                    name = L["Specialization"],
                    type = "select",
                    values = self:getSpecTable(),
                    set = function(self, value)
                        v.specialization = value
                        E.config:refreshOptions()
                    end,
                    get = function()
                        return v.specialization
                    end,
                },
                toggle = {
                    name = L["Switch talents"],
                    desc = L["If you enable this, talents for this specialization will be switched to them below if you are in a resting area."],
                    type = "toggle",
                    --width = "half",
                    get = function()
                        return v.switchTalents
                    end,
                    set = function(self, value)
                        if value then
                            v.switchTalents = true
                        else
                            v.switchTalents = false
                        end
                        E.config:refreshOptions()
                    end,
                },
                talents = {
                    name = L["Talents"],
                    type = "group",
                    disabled = not v.switchTalents,
                    inline = true,
                    args = {
                        tier1 = {
                            type = "select",
                            name = L["Tier 1"],
                            values = self:getTalentsForSpec(v.specialization or 1, 1),
                            set = function(self, value)
                                v.talents[1] = value
                            end,
                            get = function()
                                return v.talents[1]
                            end,
                        },
                        tier2 = {
                            type = "select",
                            name = L["Tier 2"],
                            values = self:getTalentsForSpec(v.specialization or 1, 2),
                            set = function(self, value)
                                v.talents[2] = value
                            end,
                            get = function()
                                return v.talents[2]
                            end,
                        },
                        tier3 = {
                            type = "select",
                            name = L["Tier 3"],
                            values = self:getTalentsForSpec(v.specialization or 1, 3),
                            set = function(self, value)
                                v.talents[3] = value
                            end,
                            get = function()
                                return v.talents[3]
                            end,
                        },
                        tier4 = {
                            type = "select",
                            name = L["Tier 4"],
                            values = self:getTalentsForSpec(v.specialization or 1, 4),
                            set = function(self, value)
                                v.talents[4] = value
                            end,
                            get = function()
                                return v.talents[4]
                            end,
                        },
                        tier5 = {
                            type = "select",
                            name = L["Tier 5"],
                            values = self:getTalentsForSpec(v.specialization or 1, 5),
                            set = function(self, value)
                                v.talents[5] = value
                            end,
                            get = function()
                                return v.talents[5]
                            end,
                        },
                        tier6 = {
                            type = "select",
                            name = L["Tier 6"],
                            values = self:getTalentsForSpec(v.specialization or 1, 6),
                            set = function(self, value)
                                v.talents[6] = value
                            end,
                            get = function()
                                return v.talents[6]
                            end,
                        },
                        tier7 = {
                            type = "select",
                            name = L["Tier 7"],
                            values = self:getTalentsForSpec(v.specialization or 1, 7),
                            set = function(self, value)
                                v.talents[7] = value
                            end,
                            get = function()
                                return v.talents[7]
                            end,
                        },
                    },
                },
                button = {
                    name = L["Activate"],
                    type = "execute",
                    order = -1,
                    --image = v.texture or 134400,
                    width = "full",
                    func = function(info)
                        E:changeTalentsAndEquipSet(info[1])
                    end,
                },
            },
        }
    end
end
local ot = {
    type = "group",
    args = {
        test1 = {
            name = "Test",
            type = "toggle",
            desc = "Hallo, test",
        }
    }
}
function E.config:init()
    self.frame = CreateFrame("Frame")
    self.frame:RegisterEvent("EQUIPMENT_SETS_CHANGED")
    self.frame:SetScript("OnEvent", function(self, event, ...)
        if event == "EQUIPMENT_SETS_CHANGED" then
            E.config:refreshOptions()
        end
    end)
    self:buildOptionsTable()
    AceConfig:RegisterOptionsTable("MultiSpec", self.optionsTable)
    AceConfigDialog:AddToBlizOptions("MultiSpec")
end

for i, v in pairs({"ms", "multispec"}) do
	_G["SLASH_MULTISPEC"..i] = "/"..v
end

function _G.SlashCmdList.MULTISPEC()
    if not AceConfigDialog.OpenFrames[addonName] then
        AceConfigDialog:Open(addonName)
    end
end
