local addonName, addonTable = ...
local L = addonTable.L
local E = addonTable.E
local tinsert = _G.tinsert
local pairs = _G.pairs
local ipairs = _G.ipairs
local table_sort = _G.table.sort
local format = _G.format
local CreateFrame = _G.CreateFrame
local GetSpecialization = _G.GetSpecialization
local GetTalentInfo = _G.GetTalentInfo
E.ui = {}

function E.ui.checkTalents(name)
    if E.equipmentSets[name].switchTalents then
        for k, v in ipairs(E.equipmentSets[name].talents) do
            local _, _, _, learned = GetTalentInfo(k, v, 1)
            if not learned then
                return false
            end
        end
    end
    return true
end

function E.ui:initScrollPaneButtons(frame)
    E:gatherEquipmentSets()
    local i = 0
    local sets = {}
    for k, v in pairs(E.equipmentSets) do
        i = i + 1
        sets[i] = {k, v.specialization, v.texture, v.missingItems, v.isEquipped, v.switchTalents}
        table_sort(sets, function(a, b)
            return a[1]<b[1]
        end)
    end
    for i=1, #frame.buttons do
        local button = frame.buttons[i]
        button:SetScript("OnEnter", function(self)
            _G.GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
            local text = L["No specialization assigned"]
            if sets[i][2]>0 then
                text = format(L["Specialization: %s"], E.specTable[sets[i][2]].name)
            end
            if not self:IsEnabled() then
                text = text .. format(L["\n%s items missing"], sets[i][4])
            end
            _G.GameTooltip:SetText(text, 1, 1, 1, true);
            self.HighlightBar:Show();
        end)
        button:SetScript("OnLeave", function(self)
            _G.GameTooltip:Hide();
            button.HighlightBar:Hide();
        end)
        button:SetScript("OnClick", function(self)
            _G.MultiSpecPaperDollPane.selectedEquipSet = self.value
            E.ui.scrollPaneUpdate(_G.MultiSpecPaperDollPane)
        end)
    end
end

function E.ui.scrollPaneUpdate(frame)
    E:gatherEquipmentSets()
    local i = 0
    local sets = {}

    for k, v in pairs(E.equipmentSets) do
        i = i + 1
        sets[i] = {k, v.specialization, v.texture, v.missingItems, v.isEquipped, v.switchTalents}
        table_sort(sets, function(a, b)
            return a[1]<b[1]
        end)
    end
    _G.HybridScrollFrame_Update(frame, #sets * frame.buttons[1]:GetHeight() + frame.EquipSet:GetHeight() + 20 , frame:GetHeight());
    local scrollOffset = _G.HybridScrollFrame_GetOffset(frame);
    for i=1, #frame.buttons do
        local button = frame.buttons[i]
        if i+scrollOffset <= #sets then
            button.text:SetText(sets[i+scrollOffset][1])
            button.value = sets[i+scrollOffset][1]
            button.icon:SetTexture(sets[i+scrollOffset][3] or 134400)
            if sets[i+scrollOffset][4] > 0 then
                button.text:SetTextColor(_G.RED_FONT_COLOR.r, _G.RED_FONT_COLOR.g, _G.RED_FONT_COLOR.b)
                button:Disable()
            else
                button.text:SetTextColor(_G.NORMAL_FONT_COLOR.r, _G.NORMAL_FONT_COLOR.g, _G.NORMAL_FONT_COLOR.b);
                button:Enable()
            end

            if sets[i+scrollOffset][5] and (sets[i+scrollOffset][2] == 0 or (sets[i+scrollOffset][2] == GetSpecialization() and E.ui.checkTalents(sets[i+scrollOffset][1]))) then
                button.Check:Show()
                --button:Disable()
            else
                button.Check:Hide()
            end

            if button.value == _G.MultiSpecPaperDollPane.selectedEquipSet then
                button.SelectedBar:Show()
                if button.Check:IsShown() then
                    _G.MultiSpecPaperDollPane.EquipSet:Disable()
                else
                    _G.MultiSpecPaperDollPane.EquipSet:Enable()
                end
            else
                button.SelectedBar:Hide()
            end

            if ((i+scrollOffset) == 1) then
                button.BgTop:Show();
                button.BgMiddle:SetPoint("TOP", button.BgTop, "BOTTOM");
            else
                button.BgTop:Hide();
                button.BgMiddle:SetPoint("TOP");
            end

            if ((i+scrollOffset) == #sets) then
                button.BgBottom:Show();
                button.BgMiddle:SetPoint("BOTTOM", button.BgBottom, "TOP");
            else
                button.BgBottom:Hide();
                button.BgMiddle:SetPoint("BOTTOM");
            end
            if ((i+scrollOffset)%2 == 0) then
                button.Stripe:SetColorTexture(0.9, 0.9, 1);
                button.Stripe:SetAlpha(0.1);
                button.Stripe:Show();
            else
                button.Stripe:Hide();
            end
        else
            button:Hide()
        end
    end
end

function E.ui:init()
    local button = CreateFrame("Button", "PaperDollSidebarTab4", _G.PaperDollSidebarTabs)
    button.id = 4
    button.GetID = function(self)
        return self.id
    end
    button:SetWidth(33)
    button:SetHeight(35)
    button:SetPoint("BOTTOMRIGHT", 0, 0)

    button.TabBg = button:CreateTexture(nil, "BACKGROUND")
    button.TabBg:SetTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
    button.TabBg:SetWidth(50)
    button.TabBg:SetHeight(43)
    button.TabBg:SetTexCoord(0.01562500, 0.79687500, 0.61328125, 0.78125000)
    button.TabBg:SetPoint("BOTTOMLEFT", -9, -2)

    button.Icon = button:CreateTexture(nil, "ARTWORK")
    button.Icon:SetTexture(134401)
    button.Icon:SetWidth(33)
    button.Icon:SetHeight(35)
    button.Icon:SetPoint("BOTTOM", 1, -2)

    button.Hider = button:CreateTexture(nil, "OVERLAY")
    button.Hider:SetTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
    button.Hider:SetWidth(34)
    button.Hider:SetHeight(19)
    button.Hider:SetTexCoord(0.01562500, 0.54687500, 0.11328125, 0.18750000)
    button.Hider:SetPoint("BOTTOM")

    button.Highlight = button:CreateTexture(nil, "HIGHLIGHT")
    button.Highlight:SetTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
    button.Highlight:SetTexCoord(0.01562500, 0.50000000, 0.19531250, 0.31640625)
    button.Highlight:SetWidth(31)
    button.Highlight:SetHeight(31)
    button.Highlight:SetPoint("TOPLEFT", 2, -3)

    button:SetScript("OnEnter", function(self)
        _G.GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        _G.GameTooltip:SetText(_G.PAPERDOLL_SIDEBARS[self:GetID()].name, 1, 1, 1);
    end)
    button:SetScript("OnLeave", function(self)
        _G.GameTooltip:Hide();
    end)
    button:SetScript("OnClick", function(self)
        _G.PaperDollFrame_SetSidebar(self, self:GetID());
    end)

    _G.PaperDollSidebarTab3:SetPoint("RIGHT", _G.PaperDollSidebarTab4, "LEFT", -4, 0)

    local frame = CreateFrame("ScrollFrame", "MultiSpecPaperDollPane", _G.PaperDollSidebarTabs, "HybridScrollFrameTemplate")
    frame:SetWidth(172)
    frame:SetHeight(354)
    frame:SetPoint("TOPLEFT", _G.CharacterFrameInsetRight, 4, -4)
    frame:SetFrameLevel(_G.CharacterFrameInsetRight:GetFrameLevel()+1)
    frame.scrollBar = CreateFrame("Slider", nil, frame, "HybridScrollBarTemplate")
    frame.scrollBar.doNotHide = 1
    frame.scrollBar:SetPoint("TOPLEFT", frame, "TOPRIGHT", 4, -13)
    frame.scrollBar:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 4, 14)
    frame.EquipSet = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
    frame.EquipSet:SetWidth(174)
    frame.EquipSet:SetHeight(22)
    frame.EquipSet:SetPoint("TOPLEFT", 0, 0)
    frame.EquipSet.ButtonBackground = frame.EquipSet:CreateTexture()
    frame.EquipSet.ButtonBackground:SetPoint("TOPLEFT", frame)
    frame.EquipSet.ButtonBackground:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, -23)
    frame.EquipSet.ButtonBackground:SetTexture(0, 0, 0)
    frame.EquipSet:SetText("Activate")
    frame.EquipSet:SetScript("OnClick", function(self)
        if frame.selectedEquipSet then
            E:changeTalentsAndEquipSet(frame.selectedEquipSet)
        end
    end)
    frame.showHook = CreateFrame("Frame", nil, frame)
    frame.showHook:SetScript("OnShow", function(self)
        frame.showHook:RegisterEvent("EQUIPMENT_SWAP_FINISHED")
        frame.showHook:RegisterEvent("PLAYER_TALENT_UPDATE")
        frame.showHook:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
        frame.showHook:RegisterEvent("EQUIPMENT_SETS_CHANGED")
        E.ui.scrollPaneUpdate(frame)
    end)
    frame.showHook:SetScript("OnHide", function(self)
        frame.showHook:UnregisterEvent("EQUIPMENT_SWAP_FINISHED")
        frame.showHook:UnregisterEvent("PLAYER_TALENT_UPDATE")
        frame.showHook:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
        frame.showHook:UnregisterEvent("EQUIPMENT_SETS_CHANGED")
    end)

    frame.showHook:SetScript("OnEvent", function(self, event, ...)
        if event == "EQUIPMENT_SWAP_FINISHED"
        or event == "PLAYER_TALENT_UPDATE"
        or event == "EQUIPMENT_SETS_CHANGED"
        or event == "PLAYER_EQUIPMENT_CHANGED" then
            E.ui.scrollPaneUpdate(frame)
        end
    end)
    _G.HybridScrollFrame_OnLoad(frame)
    frame.update = E.ui.scrollPaneUpdate
    --frame:SetHeight(600)
    _G.HybridScrollFrame_CreateButtons(frame, "GearSetButtonTemplate", 2, -(frame.EquipSet:GetHeight()+4))
    self:initScrollPaneButtons(frame)
    frame:Hide()

    tinsert(_G.PAPERDOLL_SIDEBARS, {
        name="MultiSpec";
        frame="MultiSpecPaperDollPane";
        icon = "Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs";
        texCoords = {0.01562500, 0.53125000, 0.46875000, 0.60546875};
    })

    _G.PaperDollFrame_UpdateSidebarTabs()
end
